@@ -3,45 +3,12 @@ from __future__ import annotations
 from abc import ABC, abstractmethod
 from collections.abc import Iterable
 from functools import wraps
-from typing import Any, Callable, List, Optional, Union
+from typing import Any, Callable, List, Optional, Tuple, Union, cast
 
 from pipelayer.context import Context
 from pipelayer.enum import Action, State
 from pipelayer.event_args import FilterEventArgs
-from pipelayer.protocol import FilterEventHandlerT, PipelineCallableT
-
-
-def raise_events(func: Callable) -> Callable:
-    """
-    Decorates a filter method to raise events
-    """
-    @wraps(func)
-    def wrapper(*args: Any, **kwargs: Any) -> Callable:
-        filter: Filter = args[0]
-        data: Any = args[1]
-        context: Context = args[2]
-
-        evt_args = FilterEventArgs(data, context, State.RUNNING)
-
-        filter._on_start(evt_args)
-
-        if evt_args.action in (Action.EXIT, Action.SKIP):
-            evt_args.state = State.SKIPPING if evt_args.action == Action.SKIP else State.EXITING
-            filter._on_exit(evt_args)
-            return evt_args.data
-
-        data = func(*args, **kwargs)
-
-        evt_args = FilterEventArgs(data, args[2], State.COMPLETING)
-        filter._on_end(evt_args)
-
-        if evt_args.action is Action.EXIT:
-            evt_args.state = State.EXITING
-            filter._on_exit(evt_args)
-            return evt_args.data
-
-        return data
-    return wrapper
+from pipelayer.protocol import FilterEventHandlerT, IFilter
 
 
 class EventHandlerList(List[FilterEventHandlerT]):
@@ -67,12 +34,8 @@ class Filter(ABC):
     def __init__(
         self: Filter,
         name: Optional[str] = "",
-        pre_process: Optional[PipelineCallableT] = None,
-        post_process: Optional[PipelineCallableT] = None
     ) -> None:
         self.__name = name or self.__class__.__name__
-        self.__pre_process = pre_process
-        self.__post_process = post_process
 
         self.__start: EventHandlerList = EventHandlerList()
         self.__exit: EventHandlerList = EventHandlerList()
@@ -84,14 +47,6 @@ class Filter(ABC):
     def name(self) -> str:
         return self.__name
 
-    @property
-    def pre_process(self) -> Optional[PipelineCallableT]:
-        return self.__pre_process
-
-    @property
-    def post_process(self) -> Optional[PipelineCallableT]:
-        return self.__post_process
-
     # region Event Handlers
 
     @property
@@ -146,3 +101,59 @@ class Filter(ABC):
         [e(self, args) for e in self.end]
 
     # endregion
+
+
+def _parse_args(*args: str, **kwargs: dict) -> Tuple[IFilter, Any, Context]:
+
+    if len(args) == 3:
+        return cast(IFilter, args[0]), args[1], cast(Context, args[2])
+
+    _self = None
+    _data = None
+    _context = None
+    _args: list = list(args)
+
+    if _args and isinstance(args[0], IFilter):
+        _self = cast(IFilter, _args[0])
+        _args.pop(0)
+
+    if _args:
+        _data = _args[0]
+        _args.pop(0)
+
+    if kwargs:
+        _data = _data or kwargs.get("data")
+        _context = _context or kwargs.get("context")
+
+    return cast(IFilter, _self), _data, cast(Context, _context) if _context else Context()
+
+
+def raise_events(func: Callable) -> Callable:
+    """
+    Decorates a filter method to raise events
+    """
+    @wraps(func)
+    def wrapper(*args: Any, **kwargs: Any) -> Callable:
+        filter, data, context = _parse_args(*args, **kwargs)
+
+        evt_args = FilterEventArgs(data, context, State.RUNNING)
+
+        filter._on_start(evt_args)
+
+        if evt_args.action in (Action.EXIT, Action.SKIP):
+            evt_args.state = State.SKIPPING if evt_args.action == Action.SKIP else State.EXITING
+            filter._on_exit(evt_args)
+            return evt_args.data
+
+        data = func(*args, **kwargs)
+
+        evt_args = FilterEventArgs(data, context, State.COMPLETING)
+        filter._on_end(evt_args)
+
+        if evt_args.action is Action.EXIT:
+            evt_args.state = State.EXITING
+            filter._on_exit(evt_args)
+            return evt_args.data
+
+        return data
+    return wrapper
